# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypedDict

from .chat_completion_content_part_text_param import ChatCompletionContentPartTextParam

__all__ = ["ChatCompletionPredictionContentParam"]


class ChatCompletionPredictionContentParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[ChatCompletionContentPartTextParam]]]
    """
    The content that should be matched when generating a model response. If
    generated tokens would match this content, the entire model response can be
    returned much more quickly.
    """

    type: Required[Literal["content"]]
    """The type of the predicted content you want to provide.

    This type is currently always `content`.
    """



from typing import Dict, Any
from ..._types import Stream
from .._resource import SyncAPIResource, AsyncAPIResource
from ..._return_class import HAgentListPage


class Agents(SyncAPIResource):

    @property
    def prefix(self) -> str:
        return "/agents"
    
    def list(self):
        """
        List all agents
        """
        return self._get(
            f"{self.prefix}/list_agents",
            cast_to=HAgentListPage,
        )


class AsyncAgents(AsyncAPIResource):
    """
    Asynchronous version of the Agents resource.
    """

    @property
    def prefix(self) -> str:
        return "/agents"

    async def list_agents(self):
        """
        List all agents asynchronously
        """
        return await self._get(
            f"{self.prefix}/list_agents",
            cast_to=Dict[str, Any],
        )


"""
为各个模块提供抽象类别
"""

import hai
import argparse
import os
import damei as dm
# from damei.nn.api.utils import Config
from ..utils.config_loader import PyConfigLoader as Config


class AbstractModule(object):
    name = 'default_name'
    status = 'stopped'
    description = 'default_description'
    # default_cfg = None
    cfg_meta = None
    tag = 'internal'

    def __init__(self,cfg=None, **kwargs):
        """
        初始化模块，合并配置文件
        """
        AbstractModule.status = 'ready'
        self._name = AbstractModule.name
        self.status = AbstractModule.status
        self.description = AbstractModule.description
        self.default_cfg = None
        self._mapping = None
        
        self._cfgs = None  # all configs
        self._train_cfg = None
        self._infer_cfg = None
        self._eval_cfg = None

        self._model = None
        self._device = None
        self._mode = 'train'  # mode: train, eval, infer
        
        
        # print(f'xxx {self.name}:? {self.status}')
    
    def convert_config_type(self, cfg):
        """
        conver the config type from Config object, .py file argparse.Namespace to Config object
        """
        if isinstance(cfg, Config):
            return cfg
        elif isinstance(cfg, str):
            return Config.fromfile(cfg)
        elif isinstance(cfg, dict):
            return Config.from_dict(cfg)
        elif isinstance(cfg, argparse.Namespace):
            return Config.from_argparse(cfg, name=f'{self.name}_config')
        elif isinstance(cfg, dm.misc.fake_argparse.FakeArgparse):
            return Config.from_dm_argparse(cfg, name=f'{self.name}_config')
        else:
            raise TypeError(f'{self.name}: {cfg} is not a valid config type: {type(cfg)}')
        

    def _init_cfg(self, default_cfg, cfg, **kwargs):
        """
        init config
        :param default_cfg: .py file or Config object or ArgumentParser object 
        :param cfg: .py file or Config object or 
        """
        if default_cfg is not None:
            default_cfg = self.convert_config_type(default_cfg)
                
        if cfg is not None:
            cfg = self.convert_config_type(cfg)
            # cfg = Config.fromfile(cfg) if isinstance(cfg, str) else cfg

        if default_cfg and cfg:
            cfg = default_cfg.merge(cfg2=cfg)
        elif default_cfg:
            cfg = default_cfg
        elif cfg:
            cfg = cfg
        else:  # empty config
            cfg = Config()
            # print(f'{self.name}: empty config, cfg: {cfg}')
        return cfg

    @property
    def name(self):
        if self._name == 'default_name':
            return self.__class__.__name__
        return self._name
    
    @property
    def mode(self):
        return self._mode

    @property
    def train_cfg(self):
        if self._train_cfg is None:
            raise AttributeError(f'{self.name}: train_cfg is None, please set self._train_cfg in __init__()')
        elif not isinstance(self._train_cfg, hai.Config):
            self._train_cfg = hai.Config.from_anything(self._train_cfg)
            # raise TypeError(f'{self.name}: train_cfg must be a Config object, but got {type(self._train_cfg)}')
        return self._train_cfg
    
    @property
    def eval_cfg(self):
        return self._eval_cfg
    
    @property
    def infer_cfg(self):
        return self._infer_cfg

    @property
    def cfg(self):
        """return the config of the model in current mode"""
        if self.mode == 'train':
            return self.train_cfg
        elif self.mode == 'eval':
            return self.eval_cfg
        elif self.mode == 'infer':
            return self.infer_cfg
        else:
            raise ValueError(f'{self.name}: mode "{self.mode}" is not supported')

    @property
    def cfgs(self):
        return self._cfgs

    @property
    def config(self):
        """alias of cfg"""
        return self.cfg

    @property
    def mapping(self):
        return self._mapping

    @property
    def mapped_config(self):
        if self.mapping is None:
            return self.cfg
        else:
            assert isinstance(self.mapping, dict), f'{self.name}: mapping must be a dict'
            return self.config.map_keys(self.mapping, new=True)

    @property
    def device(self):
        if self._device is None:
            import torch
            device = self.cfg.device
            if device == 'cpu':
                pass
            else:
                device = 'cuda' if torch.cuda.is_available() else 'cpu'
            self._device = device
        return self._device


    @property
    def model(self):
        if self._model is None:
            try:
                model = self.build_model()
            except:
                model = self._init_model()
            if model is not None:  # build_model有可能是返回构建的模型，也有可能是直接在函数中辅助self._model
                self._model = model
        return self._model

    def _init_model(self, *args, **kwargs):
        """init model"""
        raise NotImplementedError(f'{self.name}._init_model() not implemented, plese check the api: "{self.__module__}"')

    def build_model(self, *args, **kwargs):
        raise NotImplementedError(f'{self.name}.build_model() not implemented, plese check the api: "{self.__module__}"')
   
    def get_config(self):
        """alias of cfg"""
        return self.cfg

    def set_config(self, cfg=None, *args, **kwargs):
        """设置配置"""
        # old_cfg = self.cfg
        self.config.merge(cfg2=cfg, **kwargs)
        
        for key, value in kwargs.items():
            if key == 'weights' and value.startswith('hai/'):  # 替换成hai路径，TODO：支持下载模型权重并替换
                value = value.replace('hai/', f'{hai.config.WEIGHTS_ROOT}/')
            self.config[key] = value

    def get_cfg(self):
        return self.cfg

    def switch(self, mode):
        self._mode = mode

    def switch_train(self):
        self.switch('train')

    def switch_eval(self):
        self.switch('eval')

    def switch_infer(self):
        self.switch('infer')

    def switch_evalute(self):
        """alias of switch_eval"""
        self.switch_eval()

    def switch_inference(self):
        """alias of switch_infer"""
        self.switch_infer()

    def __call__(self, x, *args, **kwargs):
        raise NotImplementedError(f'{self.name}.__call__() not implemented, plese check the api: "{self.__module__}"')

    def forward(self, x):
        return self.model(x)

    def train(self, *args, **kwargs):
        raise NotImplementedError(f'{self.name}.train() not implemented, plese check the api: "{self.__module__}"')

    def infer(self, *args, **kwargs):
        raise NotImplementedError(f'{self.name}.infer() not implemented, plese check the api: "{self.__module__}"')

    def predict(self, *args, **kwargs):
        raise NotImplementedError(f'{self.name}.predict() not implemented, plese check the api: "{self.__module__}"')

    def evaluate(self, *args, **kwargs):
        raise NotImplementedError(f'{self.name}.evaluate() not implemented, plese check the api: "{self.__module__}"')

    def eval(self, *args, **kwargs):
        return self.evaluate(*args, **kwargs)
        
    def before_train_run(self, *args, **kwargs):
        pass

    def before_train_epoch(self, *args, **kwargs):
        pass

    def before_train_iter(self, *args, **kwargs):
        pass

    def after_train_iter(self, *args, **kwargs):
        pass

    def after_epoch(self, *args, **kwargs):
        pass

    def after_run(self, *args, **kwargs):
        pass

    




# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TextContentBlockParam"]


class TextContentBlockParam(TypedDict, total=False):
    text: Required[str]
    """Text content to be sent to the model"""

    type: Required[Literal["text"]]
    """Always `text`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    Jobs,
    AsyncJobs,
    JobsWithRawResponse,
    AsyncJobsWithRawResponse,
    JobsWithStreamingResponse,
    AsyncJobsWithStreamingResponse,
)
from .fine_tuning import (
    FineTuning,
    AsyncFineTuning,
    FineTuningWithRawResponse,
    AsyncFineTuningWithRawResponse,
    FineTuningWithStreamingResponse,
    AsyncFineTuningWithStreamingResponse,
)

__all__ = [
    "Jobs",
    "AsyncJobs",
    "JobsWithRawResponse",
    "AsyncJobsWithRawResponse",
    "JobsWithStreamingResponse",
    "AsyncJobsWithStreamingResponse",
    "FineTuning",
    "AsyncFineTuning",
    "FineTuningWithRawResponse",
    "AsyncFineTuningWithRawResponse",
    "FineTuningWithStreamingResponse",
    "AsyncFineTuningWithStreamingResponse",
]






from .worker.worker import Worker, AsyncWorker
from .api_key.api_key import Key
from .user.user import User
from .anthropic.anthropic import Anthropic
from .agents.agents import Agents, AsyncAgents


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ...chat_model import ChatModel
from ..assistant_tool_param import AssistantToolParam
from .runs.run_step_include import RunStepInclude
from .message_content_part_param import MessageContentPartParam
from ..code_interpreter_tool_param import CodeInterpreterToolParam
from ..assistant_tool_choice_option_param import AssistantToolChoiceOptionParam
from ..assistant_response_format_option_param import AssistantResponseFormatOptionParam

__all__ = [
    "RunCreateParamsBase",
    "AdditionalMessage",
    "AdditionalMessageAttachment",
    "AdditionalMessageAttachmentTool",
    "AdditionalMessageAttachmentToolFileSearch",
    "TruncationStrategy",
    "RunCreateParamsNonStreaming",
    "RunCreateParamsStreaming",
]


class RunCreateParamsBase(TypedDict, total=False):
    assistant_id: Required[str]
    """
    The ID of the
    [assistant](https://platform.openai.com/docs/api-reference/assistants) to use to
    execute this run.
    """

    include: List[RunStepInclude]
    """A list of additional fields to include in the response.

    Currently the only supported value is
    `step_details.tool_calls[*].file_search.results[*].content` to fetch the file
    search result content.

    See the
    [file search tool documentation](https://platform.openai.com/docs/assistants/tools/file-search#customizing-file-search-settings)
    for more information.
    """

    additional_instructions: Optional[str]
    """Appends additional instructions at the end of the instructions for the run.

    This is useful for modifying the behavior on a per-run basis without overriding
    other instructions.
    """

    additional_messages: Optional[Iterable[AdditionalMessage]]
    """Adds additional messages to the thread before creating the run."""

    instructions: Optional[str]
    """
    Overrides the
    [instructions](https://platform.openai.com/docs/api-reference/assistants/createAssistant)
    of the assistant. This is useful for modifying the behavior on a per-run basis.
    """

    max_completion_tokens: Optional[int]
    """
    The maximum number of completion tokens that may be used over the course of the
    run. The run will make a best effort to use only the number of completion tokens
    specified, across multiple turns of the run. If the run exceeds the number of
    completion tokens specified, the run will end with status `incomplete`. See
    `incomplete_details` for more info.
    """

    max_prompt_tokens: Optional[int]
    """The maximum number of prompt tokens that may be used over the course of the run.

    The run will make a best effort to use only the number of prompt tokens
    specified, across multiple turns of the run. If the run exceeds the number of
    prompt tokens specified, the run will end with status `incomplete`. See
    `incomplete_details` for more info.
    """

    metadata: Optional[object]
    """Set of 16 key-value pairs that can be attached to an object.

    This can be useful for storing additional information about the object in a
    structured format. Keys can be a maximum of 64 characters long and values can be
    a maximum of 512 characters long.
    """

    model: Union[str, ChatModel, None]
    """
    The ID of the [Model](https://platform.openai.com/docs/api-reference/models) to
    be used to execute this run. If a value is provided here, it will override the
    model associated with the assistant. If not, the model associated with the
    assistant will be used.
    """

    parallel_tool_calls: bool
    """
    Whether to enable
    [parallel function calling](https://platform.openai.com/docs/guides/function-calling#configuring-parallel-function-calling)
    during tool use.
    """

    response_format: Optional[AssistantResponseFormatOptionParam]
    """Specifies the format that the model must output.

    Compatible with [GPT-4o](https://platform.openai.com/docs/models#gpt-4o),
    [GPT-4 Turbo](https://platform.openai.com/docs/models#gpt-4-turbo-and-gpt-4),
    and all GPT-3.5 Turbo models since `gpt-3.5-turbo-1106`.

    Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
    Outputs which ensures the model will match your supplied JSON schema. Learn more
    in the
    [Structured Outputs guide](https://platform.openai.com/docs/guides/structured-outputs).

    Setting to `{ "type": "json_object" }` enables JSON mode, which ensures the
    message the model generates is valid JSON.

    **Important:** when using JSON mode, you **must** also instruct the model to
    produce JSON yourself via a system or user message. Without this, the model may
    generate an unending stream of whitespace until the generation reaches the token
    limit, resulting in a long-running and seemingly "stuck" request. Also note that
    the message content may be partially cut off if `finish_reason="length"`, which
    indicates the generation exceeded `max_tokens` or the conversation exceeded the
    max context length.
    """

    temperature: Optional[float]
    """What sampling temperature to use, between 0 and 2.

    Higher values like 0.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic.
    """

    tool_choice: Optional[AssistantToolChoiceOptionParam]
    """
    Controls which (if any) tool is called by the model. `none` means the model will
    not call any tools and instead generates a message. `auto` is the default value
    and means the model can pick between generating a message or calling one or more
    tools. `required` means the model must call one or more tools before responding
    to the user. Specifying a particular tool like `{"type": "file_search"}` or
    `{"type": "function", "function": {"name": "my_function"}}` forces the model to
    call that tool.
    """

    tools: Optional[Iterable[AssistantToolParam]]
    """Override the tools the assistant can use for this run.

    This is useful for modifying the behavior on a per-run basis.
    """

    top_p: Optional[float]
    """
    An alternative to sampling with temperature, called nucleus sampling, where the
    model considers the results of the tokens with top_p probability mass. So 0.1
    means only the tokens comprising the top 10% probability mass are considered.

    We generally recommend altering this or temperature but not both.
    """

    truncation_strategy: Optional[TruncationStrategy]
    """Controls for how a thread will be truncated prior to the run.

    Use this to control the intial context window of the run.
    """


class AdditionalMessageAttachmentToolFileSearch(TypedDict, total=False):
    type: Required[Literal["file_search"]]
    """The type of tool being defined: `file_search`"""


AdditionalMessageAttachmentTool: TypeAlias = Union[CodeInterpreterToolParam, AdditionalMessageAttachmentToolFileSearch]


class AdditionalMessageAttachment(TypedDict, total=False):
    file_id: str
    """The ID of the file to attach to the message."""

    tools: Iterable[AdditionalMessageAttachmentTool]
    """The tools to add this file to."""


class AdditionalMessage(TypedDict, total=False):
    content: Required[Union[str, Iterable[MessageContentPartParam]]]
    """The text contents of the message."""

    role: Required[Literal["user", "assistant"]]
    """The role of the entity that is creating the message. Allowed values include:

    - `user`: Indicates the message is sent by an actual user and should be used in
      most cases to represent user-generated messages.
    - `assistant`: Indicates the message is generated by the assistant. Use this
      value to insert messages from the assistant into the conversation.
    """

    attachments: Optional[Iterable[AdditionalMessageAttachment]]
    """A list of files attached to the message, and the tools they should be added to."""

    metadata: Optional[object]
    """Set of 16 key-value pairs that can be attached to an object.

    This can be useful for storing additional information about the object in a
    structured format. Keys can be a maximum of 64 characters long and values can be
    a maximum of 512 characters long.
    """


class TruncationStrategy(TypedDict, total=False):
    type: Required[Literal["auto", "last_messages"]]
    """The truncation strategy to use for the thread.

    The default is `auto`. If set to `last_messages`, the thread will be truncated
    to the n most recent messages in the thread. When set to `auto`, messages in the
    middle of the thread will be dropped to fit the context length of the model,
    `max_prompt_tokens`.
    """

    last_messages: Optional[int]
    """
    The number of most recent messages from the thread when constructing the context
    for the run.
    """


class RunCreateParamsNonStreaming(RunCreateParamsBase, total=False):
    stream: Optional[Literal[False]]
    """
    If `true`, returns a stream of events that happen during the Run as server-sent
    events, terminating when the Run enters a terminal state with a `data: [DONE]`
    message.
    """


class RunCreateParamsStreaming(RunCreateParamsBase):
    stream: Required[Literal[True]]
    """
    If `true`, returns a stream of events that happen during the Run as server-sent
    events, terminating when the Run enters a terminal state with a `data: [DONE]`
    message.
    """


RunCreateParams = Union[RunCreateParamsNonStreaming, RunCreateParamsStreaming]





from typing import Union, Optional, Literal, Dict, List
from pydantic import BaseModel, Field


class WorkerInfoRequest(BaseModel):
    worker_id: Optional[str] = Field(default=None, description="worker id")
    alias_model_name: Optional[str] = Field(default=None, alias="model_name", description="worker alias")


class WorkerUnifiedGateRequest(BaseModel):
    # target: dict = Field(..., description='such as {"model": "model_name", "func": "function_name"}')
    args: list = Field([], description="args")
    kwargs: dict = Field({}, description="kwargs")


class CreateUserRequest(BaseModel):
    username: str = Field(..., description="username")
    password: str = Field(default=None, description="password")
    user_level: str = Field(default='free', description="user level")
    user_groups: list = Field(default=None, description="user groups")
    email: str = Field(default=None, description="email")
    phone: str = Field(default=None, description="phone")
    auth_type: str = Field(default='local', description="auth type")
    umt_id: int = Field(default=None, description="umt id")
    cluster_uid: int = Field(default=None, description="cluster uid")
    remarks: Optional[str] = Field("", description="remarks")

class DeleteUserRequest(BaseModel):
    user_id: str = Field(..., description="user id")

class CreateAPIKeyRequest(BaseModel):
    key_name: str = Field(..., description="key name")
    valid_time: int = Field(30, description="valid time")
    allowed_models: Union[str, Dict] = Field(default="all", description="allowed models")
    user_id: Union[str, None] = Field(default=None, description="user id")
    remarks: str = Field("", description="remarks")

class DeleteAPIKeyRequest(BaseModel):
    api_key_id: str = Field(..., description="api key id")

    
    




from typing import List, Dict, Any
from pathlib import Path
here = Path(__file__).parent


class BaseWorkerModel:
    """
    用于被控制工人模型的基类，提供了面向切片的自动流式输出等功能
    Usage:
        class MyWorker(WorkerBaseModel):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
            
            @WorkerBaseModel.auto_stream  # 自动流式输出
            def inference(self, **kwargs):
                “”“重写推理函数”“”
                return "output"
            
        worker = MyWorker()
        worker.start()
    """
    def __init__(self, **kwargs) -> None:
        self.name = kwargs.pop('name', "hepai/worker-base-model")
        self.trainable = kwargs.pop('trainable', False)
        self.inferable = kwargs.pop('inferable', False)
        self.allowd_functions = ['inference', 'train', 'evaluate', "chat_completions"]

    @staticmethod
    def convert(ret):
        for x in ret:
            yield x

    @staticmethod
    def auto_stream(func):
        """
        自动流式输出
        """
        def warpper(*args, **kwargs):
            stream = kwargs.get('stream', False)    
            output = func(*args, **kwargs)
            if stream:
                is_generator =  type(output) == type((i for i in range(10)))
                if isinstance(output, str):
                    return BaseWorkerModel.convert(output)
                elif is_generator:
                    return output
                else:
                    raise TypeError(f"Output type {type(output)} is supported in stream mode, please use 'str' type.")
            else:
                return output
        return warpper
    
    def inference(self, **kwargs):
        """需要重写函数"""
        raise NotImplementedError(f"The `inference` method of `{self.name}` is not implemented.")
    
    def train(self, **kwargs):
        """需要重写函数"""
        raise NotImplementedError(f"The `train` method of `{self.name}` is not implemented.")
    
    def evaluate(self, **kwargs):
        """需要重写函数"""
        raise NotImplementedError(f"The `evaluate` method of `{self.name}` is not implemented.")
    
    def get_misc(self, **kwargs):
        """需要重写函数"""
        misc = dict()
        misc['trainable'] = self.trainable
        misc['inferable'] = self.inferable
        return misc
    
    def chat_completions(self, **kwargs):
        raise NotImplementedError(f'The `chat_completions` method of `{self.name}` is not implemented.')
    
    def get_int(self, **kwargs):
        return 1
    
    def get_float(self, **kwargs):
        return 1.0
    
    def get_bool(self, **kwargs):
        return True
    
    def get_str(self, **kwargs):
        return "string"
    
    def get_list(self, **kwargs):
        return [1, 2, 3]
    
    def get_dict(self, **kwargs):
        return {"key": "value"}
    
    def get_image(self, **kwargs):
        return Path(f"{here}/assets/demo.png")
    
    def get_pdf(self, **kwargs):
        return Path(f"{here}/assets/demo.pdf")

    def get_txt(self, **kwargs):
        return Path(f"{here}/assets/demo.txt")

    def get_stream(self, **kwargs):
        range_num = kwargs.get('range_num', 10)
        out = [i for i in range(range_num)]
        for i in out:
            yield f'data: {i}\n\n'



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

from ..file_chunking_strategy_param import FileChunkingStrategyParam

__all__ = ["FileBatchCreateParams"]


class FileBatchCreateParams(TypedDict, total=False):
    file_ids: Required[List[str]]
    """
    A list of [File](https://platform.openai.com/docs/api-reference/files) IDs that
    the vector store should use. Useful for tools like `file_search` that can access
    files.
    """

    chunking_strategy: FileChunkingStrategyParam
    """The chunking strategy used to chunk the file(s).

    If not set, will use the `auto` strategy. Only applicable if `file_ids` is
    non-empty.
    """



import os, sys
from pathlib import Path
pydir = Path(os.path.abspath(__file__)).parent

import shutil
import json


import damei as dm
# from damei.nn.api import MODULES, SCRIPTS, IOS
# from damei.nn.api import Config
# from damei.nn.api import AbstractInput, AbstractOutput, AbstractModule

from hai import MODULES, SCRIPTS, IOS, Config
from hai import AbstractModule, AbstractInput, AbstractOutput, AbstractQue

# print('visexporter')

@IOS.register_module(name='ImagesExporter')
class VisExporter(AbstractOutput):
    name = 'ImagesExporter'
    description = '图像结果输出器'

    def __init__(self, save_dir=None):
        super(VisExporter, self).__init__()
        self.save_dir = save_dir
        self._init()

    def _init(self):
        if self.save_dir is not None:
            if os.path.exists(self.save_dir):
                empty = len(os.listdir(self.save_dir)) == 0
                if empty:
                    shutil.rmtree(self.save_dir)
                else:  # 非空
                    ipt = input(f'Save dir "{self.save_dir}" not empty, remove all? [YES/no] ')
                    if ipt in ['y', 'Y', 'Yes', 'YES', '']:
                        shutil.rmtree(self.save_dir)
                    else:
                        pass
            os.makedirs(self.save_dir)

    def __call__(self, *args, **kwargs):
        pass

    def single_plot(self, img, preds, names):
        """
        单张图片绘制
        :param img:
        :param bboxes:
        :param scores:
        :param classes:
        :return:
        """
        # print('bb', bboxes)
        # print(f'scores: {scores}')
        # print(f'classes: {classes}')
        bboxes = preds[:, :4]
        scores = preds[:, 4]
        classes_idx = [int(x) for x in preds[:, 5]]
        classes = [names[int(x)] for x in classes_idx]
        colors = [(204, 0, 0), (0, 130, 255), (0, 204, 0), (0, 0, 204), (255, 130, 0)]

        for i, bbox in enumerate(bboxes):
            score = scores[i]
            cls_idx = classes_idx[i]
            cls = names[cls_idx]

            label = f'{cls} {score:.2f}'
            color = colors[cls_idx]

            trace = None
            status = None
            keypoints = None
            kp_score = None

            img = dm.general.plot_one_box_trace_pose_status(
                bbox, img,
                label=label, color=color, trace=trace, status=status,
                line_thickness=10, keypoints=keypoints, kp_score=kp_score,
                # skeleton_thickness=3, kp_radius=8, other_kp_radius=4,
        )
        return img

    def save_img(self, save_name, img):
        """
        保存图像
        """
        import cv2
        save_dir = self.save_dir if self.save_dir else '.'
        cv2.imwrite(f'{save_dir}/{save_name}', img)

    def save_json(self, path, img, pred, names):
        import numpy as np

        h, w, c = img.shape

        data = dict()
        data['version'] = 'predict'
        data['flags'] = {}
        data['imagePath'] = path
        data['imageData'] = None
        data['imageHeight'] = h
        data['imageWidth'] = w

        shapes = []
        for i, target in enumerate(pred):
            bbox = target[:4]
            score = target[4]
            cls_idx = target[5]
            cls = names[int(cls_idx)]

            shape = dict()
            shape['label'] = cls
            shape['shape_type'] = 'rectangle'
            shape['group_id'] = None
            shape['flags'] = {}

            # points = [[bbox[0], bbox[1]], [bbox[2], [bbox[3]]]]  # [x1y1x2y2] to [[x1,y1],[x2,y2]]
            points = np.array(bbox, dtype=np.int32).reshape(-1, 2).tolist()
            shape['points'] = points
            shapes.append(shape)
        data['shapes'] = shapes

        save_dir = self.save_dir if self.save_dir else '.'
        save_path = f'{save_dir}/{Path(path).stem}.json'
        with open(save_path, 'w') as f:
            json.dump(data, f, indent=4)

    def analyse_and_print_ret(self, ret, names):
        """
        ret: list, [bs]个元素，每个元素是(num_obj, 13), 13=10+3, 2: status_idx, status_name, status_score
        """
        print(f'{"-":-<30} {"Print Detection Results":^25} {"-":-<30}')
        head = ['Batch', 'Target', 'Bounding_box', 'Conf', 'Class', 'Class']
        if True:
            ptn_list = [head]
            single_ret = ret  # 单图像的
            if single_ret is None:
                ptn_list.append([0, None, None, None, None])
            else:
                sp = single_ret.shape  # shape
                num_targets = sp[0]
                for j, target in enumerate(single_ret):
                    bbox_xyxy = target[:4]
                    conf = target[4]
                    target_cls = target[5]

                    tmp = [0 + 1,
                           f'{j + 1}/{num_targets}',
                           ' '.join([str(int(x)) for x in bbox_xyxy]),
                           float(conf),
                           int(target_cls),
                           names[int(target_cls)],
                           ]

                    ptn_list.append(tmp)

            format_str = dm.misc.list2table(ptn_list, alignment="<")
            print(format_str)

        print(f'{"-":-<30} {"End Print":^25} {"-":-<30}')




import os
from pathlib import Path
from typing import Union
import argparse
from dataclasses import dataclass
import requests
import random
import string
import subprocess
import time
import re
import pwd
import sys
import warnings


@dataclass
class Config:
    gres: str 
    # not_save_changes: bool
    # partition: str
    # kvm_param: str
    nodes: int = 1
    # chdir: str = None
    # nodelist: str = None
    # exclude: str = None
    job_name: str = "auto"
    time: str = "121m"
    # daemon: bool = False
    gpu_type: str = "L40"  # options: A800, L40, K100AI
    qos: str = "gpunormal"
    debug: bool = False  # 是否开启调试模式
    
    def __post_init__(self):
        self.gpu_type_partition_map = {
            "a800": "gpu",
            "l40": "gpu",
            "k100ai": "dcu",
        }
        self.partition_qos_map = {
            "gpu": ["gpunormal", "gpudebug", "gpuintera"],
            "dcu": ["dcunormal", "dcudebug", "dcuintera", "dcudvp"],
        }
        
        self._convert_walltime()
        self._auto_job_name()
        self._check_gpu_type()
        self._check_consistency()
        
    def _auto_job_name(self):
        """自动生成job_name"""
        if self.job_name.lower() == "auto":
            job_name = f'ecs_'
            # 后面随机四个符号
            for _ in range(4):
                job_name += random.choice(string.ascii_letters).lower()
            self.job_name = job_name  
    
    def _convert_walltime(self):
        """如果time格式是121s, 121m, 121h, 121d, 则转换时间hh:mm:ss格式"""
        if isinstance(self.time, str):
            match = re.match(r'(\d+)([hmd])', self.time)
            if match:
                value, unit = match.groups()
                value = int(value)
                if unit == 'h':
                    # Convert hours to hh:mm:ss format
                    self.time = f"{value:02d}:00:00"
                elif unit == 'm':
                    hours = value // 60
                    minutes = value % 60
                    self.time = f"{hours:02d}:{minutes:02d}:00"
                elif unit == 'd':
                    self.time = f"{value * 24:02d}:00:00"
                    
    def _check_gpu_type(self):
        """检查gpu_type是否在允许的范围内"""
        allowed_gpu_types = list(self.gpu_type_partition_map.keys())
        self.gpu_type = self.gpu_type.lower()  # 转换为小写以便比较
        if self.gpu_type not in allowed_gpu_types:
            raise ValueError(f"Invalid gpu_type: {self.gpu_type}, allowed types are: {allowed_gpu_types}")
        # 根据gres自动设置gpu_type
        if self.gres.lower().startswith("dcu:"):
            if self.gpu_type != "k100ai":
                # warnings.warn(f"Using `dcu` gres, but gpu_type is set to `{self.gpu_type}`, which is not consistent with `dcu`. Setting gpu_type to `k100ai`.")
                self.gpu_type = "k100ai"

    def _check_consistency(self):
        """检查一致性"""
        p, n = self.gres.split(":")
        if p not in ["gpu", "dcu"]:
            raise ValueError(f"Invalid gres: {self.gres}, only `gpu` and `dcu` are allowed.")
        
        # 用户选择的gpu_type和gres必须一致
        if self.gpu_type_partition_map[self.gpu_type] != p:
            raise ValueError(f"gpu_type `{self.gpu_type}` is not consistent with gres `{self.gres}`.")
        
        # 队列自动更新
        allowed_qos = self.partition_qos_map.get(p, [])
        if self.qos not in allowed_qos:
            # warnings.warn(f"QOS `{self.qos}` is not in the allowed list for partition `{p}`, using default QOS.")
            # 尝试自动QOS
            p_from_qos = 'dcu' if self.qos.startswith('dcu') else 'gpu'
            new_qos = self.qos.replace(p_from_qos, p)
            if new_qos in allowed_qos:
                self.qos = new_qos
                # warnings.warn(f"QOS `{self.qos}` is set automatically to match the partition `{p}`.")
                print(f"QOS `{self.qos}` is set automatically to match the partition `{p}`.")
            else:
                raise ValueError(f"QOS `{self.qos}` is not allowed for partition `{p}`, allowed QOS are: {allowed_qos}.")    
            

def parse_args() -> Config:
    parser = argparse.ArgumentParser(description='HepAI ECS command line tool to run virtual machines.')
    parser.add_argument('command', nargs='?', default='start', choices=['start', 'stop', 'status'], help="start or stop ECS. Default is start.")
    parser.add_argument('-g', '--gres', type=str, default="gpu:1", help='Generic resource. Default is `gpu:1`, which means 1 GPU. You can also set `gpu:2`, `dcu:1`, etc.')
    parser.add_argument('-t', '--time', default="120m", help="Walltime of the machine. Default is `120m`. `m` for `minutes`, `h` for hours, `d` for days.")
    parser.add_argument('-tp', '--gpu-type', type=str, default='L40', help="Type of GPU. Default is `L40`, options: `L40`, `A800`, `K100AI`.")

    parser.add_argument('-N', '--nodes', type=int, default=1, help="Number of nodes. Default is 1.")
    parser.add_argument('-q', '--qos', type=str, default="gpunormal", help="Set Quality of Service, defualt is `gpunormal` or `dcunormal`.")
    parser.add_argument('-j', '--job-name', type=str, default="auto", help="Name of the job. Default is `auto`, which will generate a random name starting with `ecs_` and followed by 4 random letters.")
    parser.add_argument('--debug', action='store_true', help="Enable debug mode. Default is False.")
    args = parser.parse_args()
    return args

@dataclass
class NodeResource:
    num_cores: int = 64  # 64 CPU cores
    num_memory: int = 960*0.534  # 960GB
    num_cards: int = 8  # 8 GPUs
    
    
@dataclass
class SubmittedJobInfo:
    """提交作业后返回的信息"""
    jobId: str  #  872
    jobType: str  # enode
    jobPath: str  # '/.xx/Jobs/enode-20250616-211720'
    

@dataclass
class JobStatus:
    """查询后的作业状态"""
    clusterId: str  # slurm
    jobId: str  # 872
    jobStartTime: str  # '2025-06-16 21:17:20'
    jobStatus: str  # 'RUNNING'
    jobNodeList: str  # 'aigpu001'
    jobSubmitTime: str  # '2025-06-16 21:17:20'
    jobType: str  # 'enode'
    connect_sign: str  # 'True

    def __post_init__(self):
        # 将connect_sign转换为布尔值
        if self.connect_sign.lower() == 'true':
            self.connect_sign = True
        elif self.connect_sign.lower() == 'false':
            self.connect_sign = False
        else:
            raise ValueError(f"Invalid connect_sign value: {self.connect_sign}")    
    

@dataclass
class ConnectionInfo:
    """连接信息"""
    host: str
    gateway_port: int
    jobId: int

class AIEmailFetcher:
    """根据用户名，从IHEP的SSO系统获取用户的电子邮件地址"""
    
    @staticmethod
    def fetch_email(username: str) -> str:
        resp = requests.get(
            f"https://login.ihep.ac.cn/umt/api/APIafsToemail?afsAccount={username}",
            headers={
                "Content-Type": "application/json",
            }
        )
        try:
            resp.raise_for_status()  # 检查请求是否成功
            resp_json = resp.json()
        except Exception as e:
            raise RuntimeError(f"请求失败: {e}, \n{resp.text}")
        
        ret = resp_json.get("result", [])
        assert len(ret) == 1, f"Expected exactly one result, but got {len(ret)} for user `{username}`."
        
        email = ret[0].get("email", "")
        if not email:
            raise RuntimeError(f"Failed to fetch email for user `{username}`.")
        
        return email

class APIKeyFetcher:
    """根据邮箱，从HepAI平台获取API Key"""
    
    mode = "server"
    # mode = "local"
    
    
    
    def fetch_api_key(self, email: str) -> str:
        
        if self.mode == "server":
            # from hepai import HepAI
            import requests
            default_base_url = "https://aiapi.ihep.ac.cn/apiv2"
            base_url = os.getenv("HEPAI_API_BASE_URL", default_base_url)  # 可以通过环境变量覆盖默认值
            # api_key = os.getenv("HAIINK_ADMIN_API_KEY")
            # 从当前文件./INK_KEY中读取API Key
            key_file = Path(__file__).parent / "INK_KEY"
            if not key_file.exists():
                raise FileNotFoundError(f"API Key file not found: {key_file}. Please create it with your API key.")
            with open(key_file, 'r') as f:
                api_key = f.read().strip()
            if not api_key:
                raise ValueError("API Key file is empty. Please check the file content.")
            assert api_key is not None, "Please set the environment variable HAIINK_ADMIN_API_KEY with your API key."
            # client = HepAI(base_url=base_url, api_key=api_key)
            payload = {
                "username": email,
                }
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {api_key}",
                }
            resp = requests.post(
                f"{base_url}/key/fetch_api_key",
                json=payload,
                headers=headers,
            )
            try:
                resp.raise_for_status()  # 检查请求是否成功
                resp_json = resp.json()
            except Exception as e:
                raise RuntimeError(f"读取专属API Key失败: {e}, \n{resp.text}")

            key = resp_json["api_key"]
            key_alias = resp_json.get("alias", "Default")
            if not key:
                raise RuntimeError(f"Failed to fetch API Key for user `{email}`. Please check your email or contact support.")
            assert "for-haiink" in key_alias, "The fetched API Key is not for HAIINK, please check your email or contact support."
            return key
            # from hepai.types import APIKeyInfo
            # key_info: APIKeyInfo = client.fetch_api_key(username=email)
            
            # return key_info.api_key

        elif self.mode == "local":
            # 本地模式，直接从文件中读取API Key
            dir_path = f'{Path.home()}/.ink'
            if not os.path.exists(dir_path):
                os.makedirs(dir_path, exist_ok=True)
            key_file = os.path.join(dir_path, 'haiink_api_key')
            if not key_file.exists():
                raise FileNotFoundError(f"API Key file not found: {key_file}. Please create it with your API key.")
            with open(key_file, 'r') as f:
                api_key = f.read().strip()
            if not api_key:
                raise ValueError("API Key file is empty. Please check the file content.")
            return api_key
        else:
            raise ValueError(f"Invalid mode: {self.mode}. Supported modes are 'server' and 'local'.")
        
        

class HaiECS:
    """基于Slurm的Elastic Cloud Server (ECS)作业提交和管理工具"""

    def __init__(self, config) -> None:
        self.base_url = "http://aiweb02.ihep.ac.cn:8005/api"
        
        self.cfg: Config = config
        
        self.nr = NodeResource()
        self.key_fetcher = APIKeyFetcher()
        
        # 自动获取用户名
        self.username, self.uid = self._get_username()
        # self.username, self.uid = 'zdzhang', 21927
        # self.username, self.uid = 'zhangyiyu', 21628
        self.email = AIEmailFetcher.fetch_email(username=self.username)
        self.ink_token = self.key_fetcher.fetch_api_key(email=self.email)
        # print(f'Current user: {self.username}, uid: {self.uid}, email: {self.email}')
        
    def _get_username(self):
        """获取当前执行脚本的用户名"""
        user_info = pwd.getpwuid(os.getuid())
        username = user_info.pw_name
        uid = user_info.pw_uid
        if not username:
            raise RuntimeError("Failed to get the current username, please check your environment.")
        return username, uid
        
        
    def _auto_num_cpu_and_memory(self, num_cards: int):
        """根据卡的数量自动设置cpu核数和memory数量"""
        if num_cards > 8:
            raise ValueError(f"Number of cards should be no more than 8, but got {num_cards}")
        assert isinstance(num_cards, int)

        ratio = num_cards / self.nr.num_cards
        n_cores = int(self.nr.num_cores * ratio)
        n_memory = int(self.nr.num_memory * ratio)
        return n_cores, n_memory
        
        
    def poll_job_status(self, job_id: str, interval: int = 1, timeout: int = 10, max_retry_times: int = 3) -> JobStatus:
        """
        轮询查询作业状态，直到作业结束或超时
        :param job_id: 作业ID
        :param interval: 轮询间隔（秒）
        :param timeout: 超时时间（秒），默认10秒
        :return: 最终的JobStatus对象
        """
        start_time = time.time()
        
        ok_status = ['running']
        error_status = ['failed', 'cancelled', 'timeout']
        
        retry_times = 0
        while True:
            try:
                job_status_list = self.query_user_jobs()
            except Exception as e:
                if retry_times < max_retry_times:
                    retry_times += 1
                    # print(f"查询作业状态失败，正在重试... ({retry_times}/{max_retry_times})")
                    print(f"\rQuerying job status failed, retrying... ({retry_times}/{max_retry_times})", end="")
                    time.sleep(interval)
                    continue
                raise RuntimeError(f"查询作业状态失败: {e}，请联系管理员hepai@ihep.ac.cn")
            job_status = next((js for js in job_status_list if str(js.jobId) == str(job_id)), None)
            if job_status is None:
                # print(f"未找到作业 {job_id}，等待下次查询...")
                print(f"\rWaiting for the job to be running ... {time.time()-start_time:.2f}s.", end="")
            else:
                # print(f"作业 {job_id} 当前状态: {job_status.jobStatus}")
                print(f" Job (id=`{job_id}`) current status: {job_status.jobStatus}")
                if job_status.jobStatus.lower() in ok_status:
                    return job_status
                elif job_status.jobStatus.lower() in ['pending']:
                    pass  # 作业还在等待中，继续轮询
                elif job_status.jobStatus.lower() in error_status:
                    raise RuntimeError(f"作业 {job_id} 失败，状态: {job_status.jobStatus}，请联系管理员")
                else:
                    raise RuntimeError(f"作业 {job_id} 状态异常: {job_status.jobStatus}，请联系管理员")
            if time.time() - start_time > timeout:
                raise TimeoutError(f"轮询作业状态超时（{timeout}秒），作业ID: {job_id}，请手动查询或联系管理员hepai@ihep.ac.cn")
            time.sleep(interval)
            

    def get_conection_info(self, job_id: Union[str, int]) -> ConnectionInfo:
        """
curl -X GET "http://aiweb02.ihep.ac.cn:8001/api/v1/connect-job?jobId=${1}&job_type=${2}&cluster_id=slurm" \
  -H "uid: 21628" \
  -H "email: guocq@ihep.ac.cn"
  
  v2:
  curl -X GET "http://aiweb02.ihep.ac.cn:8005/api/v2/cr/connect_job?job_id=${1}&job_type=${2}&cluster_id=slurm" \
  -H "Content-Type: application/json" \
  -H "Ink-Username: zhangyiyu" \
  -H "Ink-Token: sk-PlsERgMMIdRhnytkLjNyzeybGlfOnyGZBlbKZlxMsmLrQPb"
        """
        headers = self.build_headers()
        resp = requests.get(
            f"{self.base_url}/v2/cr/connect_job?job_id={job_id}&job_type=enode&cluster_id=slurm",
            headers=headers,
        )
        try:
            resp.raise_for_status()
            resp_json = resp.json()
        except Exception as e:
            raise RuntimeError(f"请求失败: {e}")
        data = resp_json.get("data", {})
        
        
        cinfo = ConnectionInfo(**data)
        return cinfo
    
    
    def build_headers(self) -> dict:
        """
        构建请求头
        :param ink_username: Ink用户名
        :param ink_token: Ink令牌
        :return: 请求头字典
        """
        return {
            "Content-Type": "application/json",
            "Ink-Username": self.username,
            "Ink-Token": self.ink_token
        }
    
    def query_user_jobs(self) -> list[JobStatus]:
        """
       curl -X GET "http://aiweb02.ihep.ac.cn:8001/api/v1/query_job?limit=10&page=1&job_type=all&cluster_id=slurm" \
  -H "Content-Type: application/json" \
  -H "uid: 21628" \
  -H "email: guocq@ihep.ac.cn"
  
        v2:
        curl -X GET "http://aiweb02.ihep.ac.cn:8005/api/v2/cr/query_jobs?limit=10&page=1&job_type=all&cluster_id=slurm" \
  -H "Content-Type: application/json" \
  -H "Ink-Username: zhangyiyu" \
  -H "Ink-Token: sk-PlsERgMMIdRhnytkLjNyzeybGlfOnyGZBlbKZlxMsmLrQPb"
        """
        
        headers = self.build_headers()
        resp = requests.get(
            f"{self.base_url}/v2/cr/query_jobs?limit=10&page=1&job_type=all&cluster_id=slurm",
            headers=headers,
        )
        
        try:
            resp.raise_for_status()
            resp_json = resp.json()
        except Exception as e:
            raise RuntimeError(f"请求失败: {e}, \n{resp.text}")
        data = resp_json.get("data", [])
        if self.cfg.debug:
            print(f"[DEBUG] Response text: {resp.text}")
            
        job_status_list = [JobStatus(**item) for item in data.get("paginated_jobs", [])]
        return job_status_list
        
    
    
    def submit_enode_job(self):
        partition, gpu_num = self.cfg.gres.split(":")
        ncores, n_memory = self._auto_num_cpu_and_memory(int(gpu_num))
        # job_name = self._auto_job_name()
        data = {
            "job_script": "",  # 空着不写
            "job_parameters": "",  # 空着不写
            "time": self.cfg.time,  # 程序最大运行时间
            "partition": partition,  # 分区
            "nodes": str(self.cfg.nodes),  # 节点数
            "ntasks": str(ncores),  # CPU核数
            "mem": f'{n_memory}G',  # 内存
            "account": "ihepai",  # 用户的组，固定为ihepai
            "qos": self.cfg.qos,  # QOS
            "gpu_name": partition,  # gpu or dcu
            "gpu_num": gpu_num,  # gpu num
            "gpu_type": self.cfg.gpu_type,  # gpu 类型 a800 或者 l40 或者 k100ai
            "ntasks_per_node": 1,  # 不改
            "job_name": self.cfg.job_name,
            "job_type": "enode",  # 作业类型
            "cluster_id": "slurm",  # 集群ID
        }
        
        print(f"Applying Elastic Cloud Server (ECS) job with the following configuration:")
    
        prt_data = {
            "num_nodes": self.cfg.nodes,
            "cpu_cores": ncores,
            "memory": n_memory,
            "accerlerator_cards": f"{partition} * {gpu_num} ({self.cfg.gpu_type})",
        }
        length = max(len(k) for k in prt_data.keys())
        for k, v in prt_data.items():
            print(f"  {k.ljust(length)}: {v}")
        
        
        headers = self.build_headers()
        resp = requests.post(
            f"{self.base_url}/v2/cr/create_job",
            headers=headers,
            json=data
        )
        # 检查resp是否成功，失败报错
        try:
            resp.raise_for_status()
            resp_json = resp.json()
        except Exception as e:
            raise RuntimeError(f"请求失败: {e}: {resp.text}")
        
        data = resp_json.get("data", {})
        
        if self.cfg.debug:
            print(f"[DEBUG] Response text: {resp.text}")
            
        job_info = SubmittedJobInfo(**data)
        
        print(f"Job submitted, job_id: `{job_info.jobId}`")
        return job_info
    
    
    def check_existing_job(self):
        """
        检查是已经有enode作业在运行，如果有则提示用户
        """
        user_jobs = self.query_user_jobs()
        enode_jobs = [job for job in user_jobs if job.jobType.lower() == 'enode']
        if len(enode_jobs) >= 1:
            task_or_tasks = "task" if len(enode_jobs) == 1 else "tasks"
            print(f"""You already have {len(enode_jobs)} ECS {task_or_tasks} running. 
Check command: 
    `squeue` to view, 
    `scancel <job_id>` to cancel.""")
            return enode_jobs
        return None
        
    def stop_enode_job(self):
        """
        停止当前用户的enode作业（如果有）
        """
        user_jobs = self.query_user_jobs()
        enode_jobs = [job for job in user_jobs if job.jobType.lower() == 'enode']
        if not enode_jobs:
            print("No running ECS (enode) jobs found.")
            return
        for job in enode_jobs:
            print(f"Stopping ECS job: {job.jobId} ...")
            try:
                # 调用 scancel 命令
                subprocess.run(['scancel', str(job.jobId)], check=True)
                print(f"Job `{job.jobId}` stopped.")
            except Exception as e:
                print(f"Failed to stop job {job.jobId}: {e}")
        
        
    def __call__(self, check_status: bool = False) -> ConnectionInfo:
        
        enode_jobs = self.check_existing_job()
        if enode_jobs:
            # 筛选正在running的作业
            running_enode_jobs = [job for job in enode_jobs if job.jobStatus.lower() == 'running']
            if not running_enode_jobs:
                # 没有running的作业
                job_statuses = set([job.jobStatus.lower() for job in enode_jobs])
                job_stat = {}
                for js in job_statuses:
                    job_stat[js] = len([job for job in enode_jobs if job.jobStatus.lower() == js])
                print(f"Your job status: {job_stat}")
                exit(0)  # 如果没有正在运行的作业，则直接退出
            
            connection_info = self.get_conection_info(job_id=running_enode_jobs[0].jobId)
            print(f"""
The running ECS job id is `{connection_info.jobId}`, jobNodelist: `{enode_jobs[0].jobNodeList}`.
ECS information:
    HostName: {connection_info.host}
    User: {self.username}
    Port: {connection_info.gateway_port}
    
    You can connect to it via: `ssh -o UserKnownHostsFile=/dev/null {self.username}@{connection_info.host} -p {connection_info.gateway_port}`""")
            exit(0)  # 如果有正在运行的作业，则直接退出
        else:
            if check_status:
                print("No running ECS jobs found. You can start a new ECS job with the command: `hai-ecs`")
                exit(0)
            
        
        job_info = self.submit_enode_job()
        # 轮询作业状态
        time.sleep(0.5)
        final_status = self.poll_job_status(job_info.jobId)
        # if final_status.jobStatus.lower() != 'running':
        #     raise RuntimeError(f"作业 {job_info.jobId} 状态异常: {final_status.jobStatus}，请联系管理员")
        
        
        connection_info = self.get_conection_info(job_id=job_info.jobId)
        print(f"""
The ECS is ready!
    HostName: {connection_info.host}
    User: {self.username}
    Port: {connection_info.gateway_port}
    
    For more information, please visit: `https://ai.ihep.ac.cn/docs`
    You can connect to it via: `ssh -o UserKnownHostsFile=/dev/null {self.username}@{connection_info.host} -p {connection_info.gateway_port}`
    """)
        return connection_info
 
    

if __name__ == "__main__":
    args = parse_args()
    
    config = Config(
            gres=args.gres,
            nodes=args.nodes,
            qos=args.qos,
            job_name=args.job_name,
            time=args.time,
            gpu_type=args.gpu_type,
            debug=args.debug
        )
    # config.debug = True
    hai_ecs = HaiECS(config)
    
    # 判断命令
    if getattr(args, "command", "start") == "stop":
        # 只需要gres等参数用于初始化Config
        hai_ecs.stop_enode_job()
    elif getattr(args, "command", "start") == "start":
        hai_ecs()
    elif getattr(args, "command", "start") == "status":
        hai_ecs(check_status=True)
    else:
        hai_ecs()



from typing import Dict, Any
from ..._types import Stream
from .._resource import SyncAPIResource, AsyncAPIResource

from ..._return_class import (
    HWorkerListPage,
)
from ..._related_class import (
    WorkerInfo, HRemoteModel, WorkerStoppedInfo, WorkerStatusInfo,
)


class Messages(SyncAPIResource):

    @property
    def prefix(self) -> str:
        return "/anthropic"
    
    def create(self, model: str, **kwargs) -> Stream:
        """
        Create a chat completion with the given model.
        """
        payload = {
            "model": model,
            **kwargs,
        }
        return self._post(
            f"{self.prefix}/messages",
            cast_to=Stream,
            body=payload,
        )



class Anthropic(SyncAPIResource):

    @property
    def prefix(self) -> str:
        return "/anthropic"
    
    @property
    def messages(self):
        return Messages(self._client)




from __future__ import annotations

from typing import Any, Dict, cast

import pydantic

from ._pydantic import to_strict_json_schema
from ..types.chat import ChatCompletionToolParam
from ..types.shared_params import FunctionDefinition


class PydanticFunctionTool(Dict[str, Any]):
    """Dictionary wrapper so we can pass the given base model
    throughout the entire request stack without having to special
    case it.
    """

    model: type[pydantic.BaseModel]

    def __init__(self, defn: FunctionDefinition, model: type[pydantic.BaseModel]) -> None:
        super().__init__(defn)
        self.model = model

    def cast(self) -> FunctionDefinition:
        return cast(FunctionDefinition, self)


def pydantic_function_tool(
    model: type[pydantic.BaseModel],
    *,
    name: str | None = None,  # inferred from class name by default
    description: str | None = None,  # inferred from class docstring by default
) -> ChatCompletionToolParam:
    if description is None:
        # note: we intentionally don't use `.getdoc()` to avoid
        # including pydantic's docstrings
        description = model.__doc__

    function = PydanticFunctionTool(
        {
            "name": name or model.__name__,
            "strict": True,
            "parameters": to_strict_json_schema(model),
        },
        model,
    ).cast()

    if description is not None:
        function["description"] = description

    return {
        "type": "function",
        "function": function,
    }



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import FileTypes

__all__ = ["PartCreateParams"]


class PartCreateParams(TypedDict, total=False):
    data: Required[FileTypes]
    """The chunk of bytes for this Part."""



"""
Run from the repository root:
    uv run examples/snippets/clients/streamable_basic.py
"""

import asyncio

from mcp import ClientSession
from mcp.client.streamable_http import streamablehttp_client


async def main():
    # Connect to a streamable HTTP server
    # async with streamablehttp_client("http://localhost:42600/apiv2/mcp/md-6d670e6c-0ec/mcp") as (
    async with streamablehttp_client("http://localhost:42600/apiv2/mcp/md-6d670e6c-0ec/mcp") as (
    # async with streamablehttp_client("http://localhost:42601/apiv2/mcp/md-6d670e6c-0ec/mcp") as (
    # async with streamablehttp_client("http://localhost:42601/apiv2/mcp/md-6d670e6c-0ec/sse") as (
        read_stream,
        write_stream,
        _,
    ):
        # Create a session using the client streams
        async with ClientSession(read_stream, write_stream) as session:
            # Initialize the connection
            await session.initialize()
            # List available tools
            tools = await session.list_tools()
            print(f"Available tools: {[tool.name for tool in tools.tools]}")

            result = await session.call_tool("custom_method", {"a": 5, "b": 7})
            print(f"Result of custom_method(5, 7): {result}")

if __name__ == "__main__":
    asyncio.run(main())


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel
from .embedding import Embedding

__all__ = ["CreateEmbeddingResponse", "Usage"]


class Usage(BaseModel):
    prompt_tokens: int
    """The number of tokens used by the prompt."""

    total_tokens: int
    """The total number of tokens used by the request."""


class CreateEmbeddingResponse(BaseModel):
    data: List[Embedding]
    """The list of embeddings generated by the model."""

    model: str
    """The name of the model used to generate the embedding."""

    object: Literal["list"]
    """The object type, which is always "list"."""

    usage: Usage
    """The usage information for the request."""



"""
Run from the repository root:
    uvicorn examples.snippets.servers.streamable_starlette_mount:app --reload
"""

import contextlib

from starlette.applications import Starlette
from starlette.routing import Mount

from mcp.server.fastmcp import FastMCP

# Create the Echo server
echo_mcp = FastMCP(name="EchoServer", stateless_http=True)


@echo_mcp.tool()
def echo(message: str) -> str:
    """A simple echo tool"""
    return f"Echo: {message}"


# Create the Math server
math_mcp = FastMCP(name="MathServer", stateless_http=True)


@math_mcp.tool()
def add_two(n: int) -> int:
    """Tool to add two to the input"""
    return n + 2


# Create a combined lifespan to manage both session managers
@contextlib.asynccontextmanager
async def lifespan(app: Starlette):
    async with contextlib.AsyncExitStack() as stack:
        # await stack.enter_async_context(echo_mcp.session_manager.run())
        await stack.enter_async_context(math_mcp.session_manager.run())
        yield


# Create the Starlette app and mount the MCP servers
app = Starlette(
    routes=[
        Mount("/echo", echo_mcp.streamable_http_app()),
        Mount("/math", math_mcp.streamable_http_app()),
    ],
    lifespan=lifespan,
)

# Note: Clients connect to http://localhost:8000/echo/mcp and http://localhost:8000/math/mcp
# To mount at the root of each path (e.g., /echo instead of /echo/mcp):
# echo_mcp.settings.streamable_http_path = "/"
# math_mcp.settings.streamable_http_path = "/"


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app, host="0.0.0.0", port=42600)


import damei as dm
import hai

class CLIFunctions(object):

    def _deal_with_list(self, *args, **kwargs):
        """mode = list"""
        opt = self.opt
        sub_mode = opt.sub_mode
        ss_mode = opt.sub_sub_mode
        # hai list
        if sub_mode is None:
            info = self.uaii.ps(**kwargs)
        elif sub_mode == 'remote':  
            # hai list remote
            # hai list remote models
            if ss_mode is None or ss_mode in ['model', 'models']: 
                dict_info = self.uaii.list_remote_models()
                info = dm.misc.dict2info(dict_info)
            # hai list remote datasets
            elif ss_mode == 'datasets':
                dict_info = self.uaii.list_remote_datasets(sub_mode=sub_mode, ret_fmt='dict')
                # info = {}
                dict_info[f'Total {len(dict_info.keys())} datasets:'] = list(dict_info.keys())
                info = dm.misc.dict2info(dict_info)
        # hai list datasets
        elif sub_mode in ['datasets', 'dataset']:  # hai list datasets
            info = self.uaii.list_local_datasets()
            info = dm.misc.dict2info(info)
        elif sub_mode in ['models', 'model']:  # hai list models
            from hai.apis.workers_api.model import HaiModel
            ret = HaiModel.list()
            info = dm.misc.dict2info(ret)
        elif sub_mode in ['all_workers_info', 'all_workers']:
            from hai.apis.workers_api.model import HaiModel
            info = HaiModel.all_workers_info()
            info = dm.misc.dict2info(info)
            
        else:
            raise NotImplementedError(f'Not implemented sub_mode: "{sub_mode}", Please use "{hai.__appname__} -h" to see help.')
        print(info)

    def _deal_with_download(self, *args, **kwargs):
        """
        mode = download
        下载算法、数据集、预训练模型等

        addtional_params: model(default), architecture, dataset, pretrained_model
        """
        opt = self.opt
        name = opt.sub_mode
        a_param = opt.sub_sub_mode  # additonal_parameter
        assert name, f'Download name need to be specified'
        a_param = a_param if a_param else 'model'  # default download algorithm

        if a_param == 'model':
            # 匹配
            dict_info = self.uaii.list_remote_models()
            model_names = dm.misc.flatten_list([x for x in dict_info.values()])
            # model_names = [x.lower() for x in model_names]
            if name in model_names:
                self.uaii.download_model(name)
            else:
                raise NotImplementedError(f'No resource "{name}" found. opt: {opt}')

        elif a_param.startswith('arch'):
            self.uaii.download_architecture(name)
            
        elif a_param.startswith('dataset'):
            raise NotImplementedError(f'Not implemented download dataset: "{name}"')
        else:
            raise NotImplementedError(f'Not implemented download: "{name}". opt: {opt}')

    def _deal_with_start(self, *args, **kwargs):
        """
        mode = start
        example:
            >>> hai start server <port>  # default port is 9999
        """

        parser = hai.argparse.ArgumentParser()
        parser.add_argument('--mode', type=str, default='start', help='start|stop|restart mode')
        parser.add_argument('--insecure', action='store_true', help='use secure grpc or insecure grpc')
        parser.add_argument('-p', '--port', type=int, default=9999, help='port to listen on')
        parser.add_argument('--debug', '-d', default=True, action='store_true', help='debug mode')
        args = parser.parse_args()

        opt = self.opt
        server = opt.sub_mode
        port = opt.sub_sub_mode

        args.model = 'start'
        args.insecure = kwargs.get('insecure', args.insecure)
        args.port = kwargs.get('port', args.port)
        args.port = port if port else args.port  # if specified port explicitly, use it
        args.debug = kwargs.get('debug', args.debug)

        from hai.uaii.server import run as run_server
        run_server(args)

        # print(f'opt: {opt}')



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Thread", "ToolResources", "ToolResourcesCodeInterpreter", "ToolResourcesFileSearch"]


class ToolResourcesCodeInterpreter(BaseModel):
    file_ids: Optional[List[str]] = None
    """
    A list of [file](https://platform.openai.com/docs/api-reference/files) IDs made
    available to the `code_interpreter` tool. There can be a maximum of 20 files
    associated with the tool.
    """


class ToolResourcesFileSearch(BaseModel):
    vector_store_ids: Optional[List[str]] = None
    """
    The
    [vector store](https://platform.openai.com/docs/api-reference/vector-stores/object)
    attached to this thread. There can be a maximum of 1 vector store attached to
    the thread.
    """


class ToolResources(BaseModel):
    code_interpreter: Optional[ToolResourcesCodeInterpreter] = None

    file_search: Optional[ToolResourcesFileSearch] = None


class Thread(BaseModel):
    id: str
    """The identifier, which can be referenced in API endpoints."""

    created_at: int
    """The Unix timestamp (in seconds) for when the thread was created."""

    metadata: Optional[object] = None
    """Set of 16 key-value pairs that can be attached to an object.

    This can be useful for storing additional information about the object in a
    structured format. Keys can be a maximum of 64 characters long and values can be
    a maximum of 512 characters long.
    """

    object: Literal["thread"]
    """The object type, which is always `thread`."""

    tool_resources: Optional[ToolResources] = None
    """
    A set of resources that are made available to the assistant's tools in this
    thread. The resources are specific to the type of tool. For example, the
    `code_interpreter` tool requires a list of file IDs, while the `file_search`
    tool requires a list of vector store IDs.
    """



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ngrpc.proto\"3\n\x0cHelloRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12\x15\n\x05skill\x18\x02 \x01(\x0b\x32\x06.Skill\"\x84\x01\n\rHelloResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\x31\n\nmap_result\x18\x02 \x03(\x0b\x32\x1d.HelloResponse.MapResultEntry\x1a\x30\n\x0eMapResultEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"\x15\n\x05Skill\x12\x0c\n\x04name\x18\x01 \x01(\t\"+\n\x0b\x43\x61llRequest\x12\x0c\n\x04\x66unc\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\x0c\",\n\x0c\x43\x61llResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\xa4\x01\n\x06Params\x12\x1d\n\x03key\x18\x01 \x03(\x0b\x32\x10.Params.KeyEntry\x12!\n\x05value\x18\x02 \x03(\x0b\x32\x12.Params.ValueEntry\x1a*\n\x08KeyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a,\n\nValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x32^\n\x0bGrpcService\x12(\n\x05hello\x12\r.HelloRequest\x1a\x0e.HelloResponse\"\x00\x12%\n\x04\x63\x61ll\x12\x0c.CallRequest\x1a\r.CallResponse\"\x00\x42\x34\n\x1c\x63om.wxrh.cloud.grpc.sayHelloB\x0fHelloWorldProtoP\x01\x80\x01\x01\x62\x06proto3')



_HELLOREQUEST = DESCRIPTOR.message_types_by_name['HelloRequest']
_HELLORESPONSE = DESCRIPTOR.message_types_by_name['HelloResponse']
_HELLORESPONSE_MAPRESULTENTRY = _HELLORESPONSE.nested_types_by_name['MapResultEntry']
_SKILL = DESCRIPTOR.message_types_by_name['Skill']
_CALLREQUEST = DESCRIPTOR.message_types_by_name['CallRequest']
_CALLRESPONSE = DESCRIPTOR.message_types_by_name['CallResponse']
_PARAMS = DESCRIPTOR.message_types_by_name['Params']
_PARAMS_KEYENTRY = _PARAMS.nested_types_by_name['KeyEntry']
_PARAMS_VALUEENTRY = _PARAMS.nested_types_by_name['ValueEntry']
HelloRequest = _reflection.GeneratedProtocolMessageType('HelloRequest', (_message.Message,), {
  'DESCRIPTOR' : _HELLOREQUEST,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:HelloRequest)
  })
_sym_db.RegisterMessage(HelloRequest)

HelloResponse = _reflection.GeneratedProtocolMessageType('HelloResponse', (_message.Message,), {

  'MapResultEntry' : _reflection.GeneratedProtocolMessageType('MapResultEntry', (_message.Message,), {
    'DESCRIPTOR' : _HELLORESPONSE_MAPRESULTENTRY,
    '__module__' : 'grpc_pb2'
    # @@protoc_insertion_point(class_scope:HelloResponse.MapResultEntry)
    })
  ,
  'DESCRIPTOR' : _HELLORESPONSE,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:HelloResponse)
  })
_sym_db.RegisterMessage(HelloResponse)
_sym_db.RegisterMessage(HelloResponse.MapResultEntry)

Skill = _reflection.GeneratedProtocolMessageType('Skill', (_message.Message,), {
  'DESCRIPTOR' : _SKILL,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:Skill)
  })
_sym_db.RegisterMessage(Skill)

CallRequest = _reflection.GeneratedProtocolMessageType('CallRequest', (_message.Message,), {
  'DESCRIPTOR' : _CALLREQUEST,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:CallRequest)
  })
_sym_db.RegisterMessage(CallRequest)

CallResponse = _reflection.GeneratedProtocolMessageType('CallResponse', (_message.Message,), {
  'DESCRIPTOR' : _CALLRESPONSE,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:CallResponse)
  })
_sym_db.RegisterMessage(CallResponse)

Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {

  'KeyEntry' : _reflection.GeneratedProtocolMessageType('KeyEntry', (_message.Message,), {
    'DESCRIPTOR' : _PARAMS_KEYENTRY,
    '__module__' : 'grpc_pb2'
    # @@protoc_insertion_point(class_scope:Params.KeyEntry)
    })
  ,

  'ValueEntry' : _reflection.GeneratedProtocolMessageType('ValueEntry', (_message.Message,), {
    'DESCRIPTOR' : _PARAMS_VALUEENTRY,
    '__module__' : 'grpc_pb2'
    # @@protoc_insertion_point(class_scope:Params.ValueEntry)
    })
  ,
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:Params)
  })
_sym_db.RegisterMessage(Params)
_sym_db.RegisterMessage(Params.KeyEntry)
_sym_db.RegisterMessage(Params.ValueEntry)

_GRPCSERVICE = DESCRIPTOR.services_by_name['GrpcService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.wxrh.cloud.grpc.sayHelloB\017HelloWorldProtoP\001\200\001\001'
  _HELLORESPONSE_MAPRESULTENTRY._options = None
  _HELLORESPONSE_MAPRESULTENTRY._serialized_options = b'8\001'
  _PARAMS_KEYENTRY._options = None
  _PARAMS_KEYENTRY._serialized_options = b'8\001'
  _PARAMS_VALUEENTRY._options = None
  _PARAMS_VALUEENTRY._serialized_options = b'8\001'
  _HELLOREQUEST._serialized_start=14
  _HELLOREQUEST._serialized_end=65
  _HELLORESPONSE._serialized_start=68
  _HELLORESPONSE._serialized_end=200
  _HELLORESPONSE_MAPRESULTENTRY._serialized_start=152
  _HELLORESPONSE_MAPRESULTENTRY._serialized_end=200
  _SKILL._serialized_start=202
  _SKILL._serialized_end=223
  _CALLREQUEST._serialized_start=225
  _CALLREQUEST._serialized_end=268
  _CALLRESPONSE._serialized_start=270
  _CALLRESPONSE._serialized_end=314
  _PARAMS._serialized_start=317
  _PARAMS._serialized_end=481
  _PARAMS_KEYENTRY._serialized_start=393
  _PARAMS_KEYENTRY._serialized_end=435
  _PARAMS_VALUEENTRY._serialized_start=437
  _PARAMS_VALUEENTRY._serialized_end=481
  _GRPCSERVICE._serialized_start=483
  _GRPCSERVICE._serialized_end=577
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, cast
from argparse import ArgumentParser

from .._utils import get_client, print_model
from .._models import BaseModel
from .._progress import BufferReader

if TYPE_CHECKING:
    from argparse import _SubParsersAction


def register(subparser: _SubParsersAction[ArgumentParser]) -> None:
    sub = subparser.add_parser("files.create")

    sub.add_argument(
        "-f",
        "--file",
        required=True,
        help="File to upload",
    )
    sub.add_argument(
        "-p",
        "--purpose",
        help="Why are you uploading this file? (see https://platform.openai.com/docs/api-reference/ for purposes)",
        required=True,
    )
    sub.set_defaults(func=CLIFile.create, args_model=CLIFileCreateArgs)

    sub = subparser.add_parser("files.retrieve")
    sub.add_argument("-i", "--id", required=True, help="The files ID")
    sub.set_defaults(func=CLIFile.get, args_model=CLIFileCreateArgs)

    sub = subparser.add_parser("files.delete")
    sub.add_argument("-i", "--id", required=True, help="The files ID")
    sub.set_defaults(func=CLIFile.delete, args_model=CLIFileCreateArgs)

    sub = subparser.add_parser("files.list")
    sub.set_defaults(func=CLIFile.list)


class CLIFileIDArgs(BaseModel):
    id: str


class CLIFileCreateArgs(BaseModel):
    file: str
    purpose: str


class CLIFile:
    @staticmethod
    def create(args: CLIFileCreateArgs) -> None:
        with open(args.file, "rb") as file_reader:
            buffer_reader = BufferReader(file_reader.read(), desc="Upload progress")

        file = get_client().files.create(
            file=(args.file, buffer_reader),
            # casts required because the API is typed for enums
            # but we don't want to validate that here for forwards-compat
            purpose=cast(Any, args.purpose),
        )
        print_model(file)

    @staticmethod
    def get(args: CLIFileIDArgs) -> None:
        file = get_client().files.retrieve(file_id=args.id)
        print_model(file)

    @staticmethod
    def delete(args: CLIFileIDArgs) -> None:
        file = get_client().files.delete(file_id=args.id)
        print_model(file)

    @staticmethod
    def list() -> None:
        files = get_client().files.list()
        for file in files:
            print_model(file)



from .base_queue import AbstractQue


class AbstractInput(object):  # 定义它是一个可遍历的对象
    name = 'default_input_name'
    status = 'stopped'
    description = 'default_input_description'

    def __init__(self, *args, **kwargs):
        que = kwargs.pop('que', None)
        if que:
            maxlen = que.get('maxlen', 5)
            self.que = AbstractQue(maxlen, *args, **kwargs)
        else:
            self.que = None

        self._attrs = []
        self._items = dict()

        self.data = None

    @property
    def attrs(self):
        return self._attrs

    @property
    def items(self):
        return self._items

    def _init_from_PyConfigLoader(self, cfg):
        """支持dm.nn.api.Config类型的参数配置"""
        assert 'input' in cfg.keys()
        input_cfg = cfg['input']
        self._items = input_cfg
        # print(input_cfg, type(input_cfg))
        # for k, v in input_cfg.items():
        #     assert isinstance(k, str)
        #     if k not in self.attrs:
        #         self._attrs.append(k)
        #     if hasattr(self, k):
        #         delattr(self, k)
        #     setattr(self, k, v)
        #
        # print(self.attrs)
        # print(self.items)

    def __len__(self):
        return len(self.data)

    def __next__(self):
        return self.data.__next__()

    def __iter__(self):

        return self.data.__iter__()


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel
from .message_content_delta import MessageContentDelta

__all__ = ["MessageDelta"]


class MessageDelta(BaseModel):
    content: Optional[List[MessageContentDelta]] = None
    """The content of the message in array of text and/or images."""

    role: Optional[Literal["user", "assistant"]] = None
    """The entity that produced the message. One of `user` or `assistant`."""






import os, sys
from pathlib import Path
from hepai import HepAI
from dotenv import load_dotenv
here = Path(__file__).parent

load_dotenv(f'{here.parent.parent.parent}/.env')

api_key = os.getenv("ZHIZENGZENG_API_KEY")
base_url = "https://api.zhizengzeng.com/v1"

api_key = "sk-lZvDMeNaRjKjGtrLArHsVOFtfCLbJuuHWxrBuTIREHsjDqT"
base_url = "http://localhost:42602/apiv2"
model = "openai/gpt-4o"
model = "deepseek-ai/deepseek-v3"

print(f"api_key: {api_key}")
client = HepAI(api_key=api_key, base_url=base_url) # set proxy to base_url

models = client.models.list()
for m in models:
    print(f'  {m}')
print(f"total models: {len(models.data)}")


from hepai import HepAI, Stream, ChatCompletionChunk, ChatCompletion

q = "tell me story"
q = "hello"
stream = True

response: Stream = client.chat.completions.create(
    model=model,
    messages = [
        {
        "role": "user",
        "content": q,
        }
    ],
    stream=stream,
    # extra_body={"user": "test_customer"}
    )

print(f"Q: {q}")

if stream:
    print(f"R: ", end="")
    reasoning_flag = True
    for chunk in response:
        chunk: ChatCompletionChunk = chunk
        
        if chunk is None:
            continue

        if reasoning_flag:
            if not chunk.choices:
                continue
            reasoning_content = chunk.choices[0].delta.model_extra.get("reasoning_content", None)
            if reasoning_content:  # 有思考过程
                print(reasoning_content, end="", flush=True)
                continue
            if chunk.choices[0].delta.content == "\n\n":
                # 思考模式结束
                reasoning_flag = False
                print(f'A: ', end="")
                continue

        x = chunk.choices[0].delta.content
        if x:
            print(x, end="", flush=True)
        
        # print(chunk)
             
else:
    x: ChatCompletion = response
    # print(x.choices[0].message.content)

    print(x)

print()




# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .image_url_param import ImageURLParam

__all__ = ["ImageURLContentBlockParam"]


class ImageURLContentBlockParam(TypedDict, total=False):
    image_url: Required[ImageURLParam]

    type: Required[Literal["image_url"]]
    """The type of the content part."""



"""
TODO 2022.12.06
"""

import os


class Trainer(object):
    """
    hai trainner训练器
    1.收集信息：模型、数据加载器、输出器、优化器、
    2.面向切面的训练器    
    """
    
    def __init__(self, model, **kwargs):
        self.model = model
        self.kwargs = kwargs
        

    def train(self):

        for i, data in enumerate(self.model.train_loader):
            print(i, data)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Moderation", "Categories", "CategoryAppliedInputTypes", "CategoryScores"]


class Categories(BaseModel):
    harassment: bool
    """
    Content that expresses, incites, or promotes harassing language towards any
    target.
    """

    harassment_threatening: bool = FieldInfo(alias="harassment/threatening")
    """
    Harassment content that also includes violence or serious harm towards any
    target.
    """

    hate: bool
    """
    Content that expresses, incites, or promotes hate based on race, gender,
    ethnicity, religion, nationality, sexual orientation, disability status, or
    caste. Hateful content aimed at non-protected groups (e.g., chess players) is
    harassment.
    """

    hate_threatening: bool = FieldInfo(alias="hate/threatening")
    """
    Hateful content that also includes violence or serious harm towards the targeted
    group based on race, gender, ethnicity, religion, nationality, sexual
    orientation, disability status, or caste.
    """

    illicit: bool
    """
    Content that includes instructions or advice that facilitate the planning or
    execution of wrongdoing, or that gives advice or instruction on how to commit
    illicit acts. For example, "how to shoplift" would fit this category.
    """

    illicit_violent: bool = FieldInfo(alias="illicit/violent")
    """
    Content that includes instructions or advice that facilitate the planning or
    execution of wrongdoing that also includes violence, or that gives advice or
    instruction on the procurement of any weapon.
    """

    self_harm: bool = FieldInfo(alias="self-harm")
    """
    Content that promotes, encourages, or depicts acts of self-harm, such as
    suicide, cutting, and eating disorders.
    """

    self_harm_instructions: bool = FieldInfo(alias="self-harm/instructions")
    """
    Content that encourages performing acts of self-harm, such as suicide, cutting,
    and eating disorders, or that gives instructions or advice on how to commit such
    acts.
    """

    self_harm_intent: bool = FieldInfo(alias="self-harm/intent")
    """
    Content where the speaker expresses that they are engaging or intend to engage
    in acts of self-harm, such as suicide, cutting, and eating disorders.
    """

    sexual: bool
    """
    Content meant to arouse sexual excitement, such as the description of sexual
    activity, or that promotes sexual services (excluding sex education and
    wellness).
    """

    sexual_minors: bool = FieldInfo(alias="sexual/minors")
    """Sexual content that includes an individual who is under 18 years old."""

    violence: bool
    """Content that depicts death, violence, or physical injury."""

    violence_graphic: bool = FieldInfo(alias="violence/graphic")
    """Content that depicts death, violence, or physical injury in graphic detail."""


class CategoryAppliedInputTypes(BaseModel):
    harassment: List[Literal["text"]]
    """The applied input type(s) for the category 'harassment'."""

    harassment_threatening: List[Literal["text"]] = FieldInfo(alias="harassment/threatening")
    """The applied input type(s) for the category 'harassment/threatening'."""

    hate: List[Literal["text"]]
    """The applied input type(s) for the category 'hate'."""

    hate_threatening: List[Literal["text"]] = FieldInfo(alias="hate/threatening")
    """The applied input type(s) for the category 'hate/threatening'."""

    illicit: List[Literal["text"]]
    """The applied input type(s) for the category 'illicit'."""

    illicit_violent: List[Literal["text"]] = FieldInfo(alias="illicit/violent")
    """The applied input type(s) for the category 'illicit/violent'."""

    self_harm: List[Literal["text", "image"]] = FieldInfo(alias="self-harm")
    """The applied input type(s) for the category 'self-harm'."""

    self_harm_instructions: List[Literal["text", "image"]] = FieldInfo(alias="self-harm/instructions")
    """The applied input type(s) for the category 'self-harm/instructions'."""

    self_harm_intent: List[Literal["text", "image"]] = FieldInfo(alias="self-harm/intent")
    """The applied input type(s) for the category 'self-harm/intent'."""

    sexual: List[Literal["text", "image"]]
    """The applied input type(s) for the category 'sexual'."""

    sexual_minors: List[Literal["text"]] = FieldInfo(alias="sexual/minors")
    """The applied input type(s) for the category 'sexual/minors'."""

    violence: List[Literal["text", "image"]]
    """The applied input type(s) for the category 'violence'."""

    violence_graphic: List[Literal["text", "image"]] = FieldInfo(alias="violence/graphic")
    """The applied input type(s) for the category 'violence/graphic'."""


class CategoryScores(BaseModel):
    harassment: float
    """The score for the category 'harassment'."""

    harassment_threatening: float = FieldInfo(alias="harassment/threatening")
    """The score for the category 'harassment/threatening'."""

    hate: float
    """The score for the category 'hate'."""

    hate_threatening: float = FieldInfo(alias="hate/threatening")
    """The score for the category 'hate/threatening'."""

    illicit: float
    """The score for the category 'illicit'."""

    illicit_violent: float = FieldInfo(alias="illicit/violent")
    """The score for the category 'illicit/violent'."""

    self_harm: float = FieldInfo(alias="self-harm")
    """The score for the category 'self-harm'."""

    self_harm_instructions: float = FieldInfo(alias="self-harm/instructions")
    """The score for the category 'self-harm/instructions'."""

    self_harm_intent: float = FieldInfo(alias="self-harm/intent")
    """The score for the category 'self-harm/intent'."""

    sexual: float
    """The score for the category 'sexual'."""

    sexual_minors: float = FieldInfo(alias="sexual/minors")
    """The score for the category 'sexual/minors'."""

    violence: float
    """The score for the category 'violence'."""

    violence_graphic: float = FieldInfo(alias="violence/graphic")
    """The score for the category 'violence/graphic'."""


class Moderation(BaseModel):
    categories: Categories
    """A list of the categories, and whether they are flagged or not."""

    category_applied_input_types: CategoryAppliedInputTypes
    """
    A list of the categories along with the input type(s) that the score applies to.
    """

    category_scores: CategoryScores
    """A list of the categories along with their scores as predicted by model."""

    flagged: bool
    """Whether any of the below categories are flagged."""



"""
v2.0 基于HClient的客户端
"""
from dataclasses import dataclass, field, asdict


from .hclient import HClient, HClientConfig, AsyncHClient
from .hclient import resources

from .hclient._related_class import HRemoteModel, WorkerInfo, WorkerStatusInfo, WorkerStoppedInfo
from .hclient._remote_model import LRemoteModel



@dataclass
class HepAIClientConfig(HClientConfig):
    base_url: str = "https://aiapi.ihep.ac.cn/apiv2"


class HepAIClient(HClient):
    """
    HepAI Client v2.0

    Usage:
        ```python
        client = HepAIClient()
        client.list_workers()
        ```
    """

    def __init__(
            self, 
            config: HClientConfig = None,
            **overrides,
            ):
        config = config or HClientConfig()
        super().__init__(config, **overrides)

        self.worker = resources.Worker(self)
        self.key = resources.Key(self)
        self.user = resources.User(self)
        self.agents = resources.Agents(self)

     ## --- 关于Worker的函数 --- ## 
    def list_workers(self):
        return self.worker.list_workers()
    
    def get_worker_info(
            self, 
            worker_id: str = None, 
            model_name: str = None,
            ) -> WorkerInfo:
        """
        Get worker info by worker_id via http request
    
        Args:
            worker_id (str): Worker ID
        """
        return self.worker.get_info(worker_id=worker_id, model_name=model_name)
    
    def get_worker_status(self, worker_id: str, refresh: bool = False) -> WorkerStatusInfo:
        """
        Get worker status by worker_id via http request
        
        Args:
            worker_id (str): Worker ID
            refresh (bool): Whether to refresh the worker status
        """
        return self.worker.get_status(worker_id=worker_id, refresh=refresh)

    def stop_worker(self, worker_id: str, permanent: bool = False) -> WorkerStoppedInfo:
        """
        Stop worker by worker_id via http request
        
        Args:
            worker_id (str): Worker ID
            permanent (bool): Whether to stop the worker permanently
        """
        return self.worker.stop(worker_id=worker_id, permanent=permanent)

    def refresh_all_workers(self) -> dict:
        """
        Refresh all workers
        """
        return self.worker.refresh_all()
    
    def get_remote_model(self, model_name: str) -> LRemoteModel:
        """
        Get a remote model
        """
        return self.worker.get_remote_model(model_name=model_name)
    
    def connect_remote_model(self, model_name: str) -> LRemoteModel:
        """
        Connect to a remote model
        """
        return self.worker.get_remote_model(model_name=model_name)
    
    def connect_to(self, model_name: str) -> LRemoteModel:
        """
        alias Connect to a remote model
        """
        return self.connect_remote_model(model_name=model_name)
    
    def request_worker(
            self,
            target: dict,
            args: list = None,
            kwargs: dict = None,
            ):
        """
        Request worker via http request

        Args:
            target (dict): Target worker and function, for example: {"model": "HRemoteModel", "function": "hello_world"}
            args (list): Arguments, can by any thing you defined in your `HRemoteModel`
            kwargs (dict): Keyword arguments, can by any thing you defined in your `HRemoteModel`
        """
        return self.worker.request(target=target, args=args, kwargs=kwargs)

    def register_worker(
            self,
            model: HRemoteModel,
            daemon: bool = False,
            standalone: bool = False,
            ):
        """
        注册Worker到服务端

        Args:
            model (HRemoteModel): Worker模型
            daemon (bool): 是否以守护进程方式启动
            standalone (bool): 是否以独立进程方式启动
        """
        return self.worker.register(model, daemon=daemon, standalone=standalone)
    
    def run_demo_worker_and_register(
            self,
        ):
        return self.worker.register(standalone=True)

    
    ## # --- 关于User的函数 --- ##
    def list_users(self):
        """
        List all users
        Note: Only admin can use this function
        """
        return self.user.list_users()
    
    def create_user(self, **kwargs):
        """
        Create a new user
        Note: Only admin can use this function
        """
        return self.user.create_user(**kwargs)
    
    def delete_user(self, user_id: str):
        """
        Delete a user
        Note: Only admin can use this function
        """
        return self.user.delete_user(user_id=user_id)
    
    def auth_user(self, username: str, password: str):
        """
        Auth a user by username and password
        """
        return self.user.auth_user(username=username, password=password)
    
    ### --- 关于Key的函数 --- ###
    def list_api_keys(self):
        """
        List all keys
        Note: Only admin can use this function
        """
        return self.key.list_api_keys()
    
    def create_api_key(self, **kwargs):
        """
        Create a new key
        Note: Only admi and app admin can use this function
        Args:
            key_name (str): Key name, default is "Default"
            valid_time (int): Valid time (days), default is 30
            user_id (str): User ID, default is None, means create a key for myself
            allowed_models (Union[str, Dict]): Allowed models, default is "all"
            remarks (str): Remarks, default is ""
        """
        return self.key.create_api_key(**kwargs)
    
    def delete_api_key(self, api_key_id: str):
        """
        Delete a key
        Note: Only admin can use this function
        """
        return self.key.delete_api_key(api_key_id=api_key_id)
    
    def verify_api_key(self, api_key: str, version: str = "v2"):
        return self.key.get_info(api_key=api_key, version = version)
    
    def fetch_api_key(self, username: str):
        """
        Fetch API Key by username
        """
        return self.key.fetch_api_key(username=username)
    
    ## --- 关于Agents的函数 --- ##
    def list_agents(self):
        """
        List all agents
        """
        return self.agents.list_agents()

class AsyncHepAIClient(AsyncHClient):
    """
    HepAI Client v2.0 (Asynchronous)

    Usage:
        ```python
        client = AsyncHepAIClient()
        await client.list_workers()
        ```
    """

    def __init__(
            self, 
            config: HClientConfig = None,
            **overrides,
            ):
        config = config or HClientConfig()
        super().__init__(config, **overrides)

        self.worker = resources.AsyncWorker(self)
        self.agents = resources.AsyncAgents(self)

        # TODO: AsyncUser and AsyncKey
        # self.key = resources.AsyncKey(self)
        # self.user = resources.AsyncUser(self)

    ## --- 关于Worker的函数 --- ## 
    async def list_workers(self):
        return await self.worker.list_workers()
    
    async def get_worker_info(
            self, 
            worker_id: str = None, 
            model_name: str = None,
            ) -> WorkerInfo:
        """
        Get worker info by worker_id via http request
        """
        return await self.worker.get_info(worker_id=worker_id, model_name=model_name)
    
    async def get_worker_status(self, worker_id: str, refresh: bool = False) -> WorkerStatusInfo:
        """
        Get worker status by worker_id via http request
        """
        return await self.worker.get_status(worker_id=worker_id, refresh=refresh)

    async def stop_worker(self, worker_id: str, permanent: bool = False) -> WorkerStoppedInfo:
        """
        Stop worker by worker_id via http request
        """
        return await self.worker.stop(worker_id=worker_id, permanent=permanent)

    async def refresh_all_workers(self) -> dict:
        """
        Refresh all workers
        """
        return await self.worker.refresh_all()
    
    async def get_remote_model(self, model_name: str) -> LRemoteModel:
        """
        Get a remote model
        """
        return await self.worker.get_remote_model(model_name=model_name)
    
    async def request_worker(
            self,
            target: dict,
            args: list = None,
            kwargs: dict = None,
            ):
        """
        Request worker via http request
        """
        return await self.worker.request(target=target, args=args, kwargs=kwargs)

    async def register_worker(
            self,
            model: HRemoteModel,
            daemon: bool = False,
            standalone: bool = False,
            ):
        """
        Register a worker to the server
        """
        return await self.worker.register(model, daemon=daemon, standalone=standalone)
    
    async def run_demo_worker_and_register(self):
        return await self.worker.register(standalone=True)

    ## # --- 关于User的函数 --- ##
    async def list_users(self):
        """
        List all users
        Note: Only admin can use this function
        """
        return await self.user.list_users()
    
    async def create_user(self, **kwargs):
        """
        Create a new user
        Note: Only admin can use this function
        """
        return await self.user.create_user(**kwargs)
    
    async def delete_user(self, user_id: str):
        """
        Delete a user
        Note: Only admin can use this function
        """
        return await self.user.delete_user(user_id=user_id)
    
    async def auth_user(self, username: str, password: str):
        """
        Auth a user by username and password
        """
        return await self.user.auth_user(username=username, password=password)

    ### --- 关于Key的函数 --- ###
    async def list_api_keys(self):
        """
        List all keys
        Note: Only admin can use this function
        """
        return await self.key.list_api_keys()
    
    async def create_api_key(self, **kwargs):
        """
        Create a new key
        Note: Only admin and app admin can use this function
        """
        return await self.key.create_api_key(**kwargs)
    
    async def delete_api_key(self, api_key_id: str):
        """
        Delete a key
        Note: Only admin can use this function
        """
        return await self.key.delete_api_key(api_key_id=api_key_id)
    
    async def verify_api_key(self, api_key: str, version: str = "v2"):
        return await self.key.get_info(api_key=api_key, version=version)
    
    ## --- 关于Agents的函数 --- ##
    async def list_agents(self):
        """
        List all agents asynchronously
        """
        return await self.agents.list_agents()


import os, sys
from typing import Dict, Union, Literal, List, Generator, Optional
from pydantic import BaseModel
from dataclasses import dataclass, field
import json
from pathlib import Path
here = Path(__file__).parent
try:
    from hepai import __version__
except:
    sys.path.insert(1, f'{here.parent.parent.parent}')
    from hepai import __version__
import hepai as hai
from hepai import HepAI, AsyncHepAI
from hepai import HRModel, HModelConfig, HWorkerConfig, HWorkerAPP
from hepai.types import ChatCompletion, ChatCompletionChunk
import uvicorn
from fastapi import FastAPI
import threading, time

    

class ChatMessage(BaseModel):
    role: Literal["user", "assistant", "system", "function"]
    content: Optional[str | List]
    function_call: Optional[Dict] = None

class ChatCompletionRequest(BaseModel):
    engine: Optional[str] = None
    messages: List[ChatMessage]
    functions: Optional[List[Dict]] = None
    temperature: Optional[float] = None  # between 0 and 2    Defaults to 1
    top_p: Optional[float] = None  # Defaults to 1
    max_length: Optional[int] = None
    stream: Optional[bool] = False
    stop: Optional[List[str]] = None


class WorkerModel(HRModel):  # Define a custom worker model inheriting from HRModel.
    def __init__(self, config: "ModelConfig"):
        super().__init__(config=config)
        self.cfg = config
        self._client = None
        self.is_o1 = self.judge_is_o1()
        
        if self.cfg.test:
            self.run_test()


    @property
    def client(self):
        if self._client is None:
            if self.cfg.use_async:
                self._client = AsyncHepAI(
                    base_url=self.cfg.base_url,
                    api_key=self.cfg.api_key,
                    proxy=self.cfg.proxy
                )
            else:
                self._client = HepAI(
                    base_url=self.cfg.base_url,
                    api_key=self.cfg.api_key,
                    proxy=self.cfg.proxy
                )
        return self._client

    @property
    def oai_param_keys(self):
        return [
            "messages", "model", "frequency_penalty",  "function_call", "functions", "logit_bias", "logprobs", "max_tokens", "n", 
            "presence_penalty", "response_format", "seed", "stop", "stream", "stream_options", 
            "temperature", "tool_choice", "tools", "top_logprobs", "top_p", "user", "extra_headers", "extra_query", "extra_body", "timeout"]
    
    def judge_is_o1(self):
        if "/" in self.cfg.engine:
            m = self.cfg.engine.split("/")[1]
        else:
            m = self.cfg.engine
        if m in ["o1", "o1-mini", "o1-preview"]:
            return True
        return False

    def response_to_stream(self, response):
        for chunk in response:
            chunk_data = chunk.model_dump()
            if chunk_data["choices"]:
                yield f'data: {json.dumps(chunk_data)}\n\n'

    def request_openai(
            self, 
            oai_messages: List,
            stream: bool = False,
            extra_headers: None = None,
            **kwargs):
        oai_params = {k: v for k, v in kwargs.items() if k in self.oai_param_keys}
        oai_params.pop("model", None)
        oai_params.pop("messages", None)
        extra_body: Dict = oai_params.pop("extra_body", {})
        
        response = self.client.chat.completions.create(
            model=self.cfg.engine, 
            messages=oai_messages, 
            stream=stream,
            extra_headers=extra_headers,
            extra_body=extra_body,
            **oai_params
            )
        return response
    

    def run_test(self):

        params = dict()
        q = "Sai hello"
        params['messages'] = [
                {
                "role": "user",
                "content": q,
                }
            ]
        params['stream'] = True
        print(f"Q: {q}")
        print(f"R: ", end="")
        reasoning_flag = True

        response = self.chat_completions(**params)
        for chunk in response:
            chunk: ChatCompletionChunk = chunk
            if isinstance(chunk, str) and chunk.startswith('data: '):
                # chunk = dict(chunk[6::])
                chunk = json.loads(chunk[6::])
                chunk = ChatCompletionChunk(**chunk)
            if reasoning_flag:
                reasoning_content = chunk.choices[0].delta.model_extra.get("reasoning_content", None)
                if reasoning_content:  # 有思考过程
                    print(reasoning_content, end="", flush=True)
                    continue
                if chunk.choices[0].delta.content == "\n\n":
                    # 思考模式结束
                    reasoning_flag = False
                    print(f'A: ', end="")
                    continue
            print('chunk', chunk, type(chunk))
            print('chunk.choices[0]', chunk.choices[0], type(chunk.choices[0]))
            print('chunk.choices[0].delta', chunk.choices[0].delta, type(chunk.choices[0].delta))
            print('chunk.choices[0].delta.content', chunk.choices[0].delta.content, type(chunk.choices[0].delta.content))
            x = chunk.choices[0].delta.content
            print('x', x, type(x))
            if x:
                print(x, end="", flush=True)
        

    @HRModel.remote_callable
    def chat_completions(self, *args, **kwargs):
        # 请求litellm时需要携带api-key
        if self.cfg.need_external_api_key:
            api_key = kwargs.pop("api_key", None)
            if not api_key:
                raise KeyError("You should provied API-KEY when calling this worker")
            extra_headers = {"Authorization": f"Bearer {api_key}"}
        else:
            extra_headers = kwargs.pop("extra_headers", {})

        request = ChatCompletionRequest(**kwargs)

        #提取request中的message，生成符合openai格式的message
        oai_messages = [{"role": msg.role, "content": msg.content} for msg in request.messages] 
        kwargs = {k: v for k, v in kwargs.items() if k not in ['model', 'messages', 'stream']}

        if self.is_o1:  # o1不允许tempterature、top_p等参数
            kwargs.pop("temperature", None)
            kwargs.pop("top_p", None)

        if request.stream:
            response = self.request_openai(
                oai_messages=oai_messages,
                stream=True,
                extra_headers=extra_headers,
                **kwargs
            )
            gen: Generator = self.response_to_stream(response)
            return gen
        else:
            response = self.request_openai(
                oai_messages=oai_messages,
                stream=False,
                extra_headers=extra_headers,
                **kwargs,
            )
            return response
        
        
@dataclass
class ModelConfig(HModelConfig):
    name: str = field(default="hepai/deepseek-r1:32b", metadata={"help": "Model's name"})
    permission: Union[str, Dict] = field(default=None, metadata={"help": "Model's permission, separated by ;, e.g., 'groups: all; users: a, b; owner: c', will inherit from worker permissions if not setted"})
    version: str = field(default="2.0", metadata={"help": "Model's version"})
    
    engine: str = field(default="deepseek-r1:32b", metadata={"help": "Model engine"})
    base_url: str = field(default="<base_url>", metadata={"help": "Base url of the litellm"})
    _api_key: str = field(default=HepAI.NotGiven, metadata={"help": "API key of the model"})
    proxy: str = field(default=None, metadata={"help": "Proxy of the model"})
    need_external_api_key: bool = field(default=False, metadata={"help": "Need external api key from user，就是每次发送请求都需要外部传输过来"})
    use_async: bool = field(default=False, metadata={"help": "whether use async client"})
    test: bool = field(default=True, metadata={"help": "Test model"})

    def __post_init__(self):
        if isinstance(self._api_key, str) and self._api_key.startswith("os.environ/"):
            environ_name = self._api_key.split("/")[1]
            self._api_key = os.getenv(environ_name)
        self.api_key = self._api_key

@dataclass
class WorkerConfig(HWorkerConfig):
    host: str = field(default="0.0.0.0", metadata={"help": "Worker's address, enable to access from outside if set to `0.0.0.0`, otherwise only localhost can access"})
    port: int = field(default=0, metadata={"help": "Worker's port, default is None, which means auto start from `auto_start_port`"})
    auto_start_port: int = field(default=42602, metadata={"help": "Worker's start port, only used when port is set to `auto`"})
    route_prefix: str = field(default="/apiv2", metadata={"help": "Route prefix for worker"})

    controller_address: str = field(default="http://aiapi.ihep.ac.cn:42601", metadata={"help": "Controller's address"})
    no_register: bool = field(default=False, metadata={"help": "Do not register to controller"})
    permissions: str = field(default='users: admin; groups: default', metadata={"help": "Model's permissions, separated by ;, e.g., 'groups: default; users: a, b; owner: c'"})
    description: str = field(default='This is a demo worker of HEP AI framework (HepAI)', metadata={"help": "Model's description"})
    author: str = field(default=None, metadata={"help": "Model's author"})
    num_workers: int = field(default=1, metadata={"help": "Number of workers"})
    daemon: bool = field(default=True, metadata={"help": "Run as daemon"})
    

    # worker_id: str = field(default=None, metadata={"help": "Worker's id"})

def get_uuid(lenth, prefix=None):
    import uuid
    if prefix:
        return prefix + str(uuid.uuid4())[:lenth-len(prefix)]
    return str(uuid.uuid4())[:lenth]
def start_uvicorn(app, host, port):
    uvicorn.run(app, host=host, port=port)


def run_worker():
    model_config, worker_config = hai.parse_args((ModelConfig, WorkerConfig))
    model = WorkerModel(model_config)  # Instantiate the custom worker model.
    
    if worker_config.num_workers == 1:
        app: FastAPI = HWorkerAPP(model, worker_config=worker_config)  # Instantiate the APP, which is a FastAPI application.
        print(app.worker.get_worker_info(), flush=True)
        start_uvicorn(app, app.host, app.port)
    else:
        n = worker_config.num_workers
        threads = []
        for i in range(n):
            w_id = get_uuid(lenth=15, prefix="wk-")
            w_port = worker_config.auto_start_port + i
            app: FastAPI = HWorkerAPP(
                model, worker_config=worker_config, 
                worker_id=w_id, port=w_port)
            print(app.worker.get_worker_info(), flush=True)
            thread = threading.Thread(target=start_uvicorn, args=(app, app.host, app.port))
            thread.start()
            threads.append(thread)
            time.sleep(0.1)
        for t in threads:
            t.join()


if __name__ == "__main__":
    # print(app.worker.get_worker_info(), flush=True)
    run_worker()
    # 启动服务
    # uvicorn.run(app, host=app.host, port=app.port)
    # uvicorn.run("dpsk_worker:app", host=app.host, port=app.port, workers=worker_config.num_workers)
    
